@@ -68,6 +68,9 @@ public:
 	INPUT *get_input() { return &this->input; }
 	OUTPUT *get_output() { return &this->output; }
 
+	const INPUT *get_input() const { return &this->input; }
+	const OUTPUT *get_output() const { return &this->output; }
+
 public:
 	void *user_data;
 
@@ -134,6 +137,9 @@ public:
 	REQ *get_req() { return &this->req; }
 	RESP *get_resp() { return &this->resp; }
 
+	const REQ *get_req() const { return &this->req; }
+	const RESP *get_resp() const { return &this->resp; }
+
 public:
 	void *user_data;
 
@@ -207,6 +213,12 @@ public:
 		}
 	}
 
+public:
+	void set_prepare(std::function<void (WFNetworkTask<REQ, RESP> *)> prep)
+	{
+		this->prepare = std::move(prep);
+	}
+
 public:
 	void set_callback(std::function<void (WFNetworkTask<REQ, RESP> *)> cb)
 	{
@@ -226,6 +238,7 @@ protected:
 	int watch_timeo;
 	REQ req;
 	RESP resp;
+	std::function<void (WFNetworkTask<REQ, RESP> *)> prepare;
 	std::function<void (WFNetworkTask<REQ, RESP> *)> callback;
 
 protected:
@@ -325,6 +338,8 @@ public:
 public:
 	ARGS *get_args() { return &this->args; }
 
+	const ARGS *get_args() const { return &this->args; }
+
 	long get_retval() const
 	{
 		if (this->state == WFT_STATE_SUCCESS)
@@ -749,6 +764,7 @@ public:
 		this->create = std::move(create);
 	}
 
+public:
 	void set_callback(std::function<void (WFRepeaterTask *)> cb)
 	{
 		this->callback = std::move(cb);
