@@ -19,6 +19,7 @@
 #ifndef _IOREQUEST_H_
 #define _IOREQUEST_H_
 
+#include <errno.h>
 #include "SubTask.h"
 #include "Communicator.h"
 
@@ -34,11 +35,7 @@ public:
 	virtual void dispatch()
 	{
 		if (this->service->request(this) < 0)
-		{
-			this->state = IOS_STATE_ERROR;
-			this->error = errno;
-			this->subtask_done();
-		}
+			this->handle(IOS_STATE_ERROR, errno);
 	}
 
 protected:
