@@ -39,6 +39,12 @@ public:
 			this->handle(SS_STATE_ERROR, errno);
 	}
 
+protected:
+	int cancel()
+	{
+		return this->scheduler->unsleep(this);
+	}
+
 protected:
 	int state;
 	int error;
