@@ -22,20 +22,14 @@ void SubTask::subtask_done()
 {
 	SubTask *cur = this;
 	ParallelTask *parent;
-	SubTask **entry;
 
 	while (1)
 	{
 		parent = cur->parent;
-		entry = cur->entry;
 		cur = cur->done();
 		if (cur)
 		{
 			cur->parent = parent;
-			cur->entry = entry;
-			if (parent)
-				*entry = cur;
-
 			cur->dispatch();
 		}
 		else if (parent)
@@ -62,7 +56,6 @@ void ParallelTask::dispatch()
 		do
 		{
 			(*p)->parent = this;
-			(*p)->entry = p;
 			(*p)->dispatch();
 		} while (++p != end);
 	}
