@@ -137,13 +137,12 @@ static inline int __poller_close_timerfd(int fd)
 
 static inline int __poller_add_timerfd(int fd, poller_t *poller)
 {
-	struct epoll_event ev = {
-		.events		=	EPOLLIN | EPOLLET,
+	static struct poller_result node = {
 		.data		=	{
-			.ptr	=	NULL
+			.operation	=	PD_OP_TIMER
 		}
 	};
-	return epoll_ctl(poller->pfd, EPOLL_CTL_ADD, fd, &ev);
+	return __poller_add_fd(fd, EPOLLIN | EPOLLET, &node, poller);
 }
 
 static inline int __poller_set_timerfd(int fd, const struct timespec *abstime,
@@ -224,6 +223,11 @@ static inline int __poller_add_timerfd(int fd, poller_t *poller)
 static int __poller_set_timerfd(int fd, const struct timespec *abstime,
 								poller_t *poller)
 {
+	static struct poller_result node = {
+		.data		=	{
+			.operation	=	PD_OP_TIMER
+		}
+	};
 	struct timespec curtime;
 	long long nseconds;
 	struct kevent ev;
@@ -231,18 +235,20 @@ static int __poller_set_timerfd(int fd, const struct timespec *abstime,
 
 	if (abstime->tv_sec || abstime->tv_nsec)
 	{
+		flags = EV_ADD | EV_ONESHOT;
 		clock_gettime(CLOCK_MONOTONIC, &curtime);
 		nseconds = 1000000000LL * (abstime->tv_sec - curtime.tv_sec);
 		nseconds += abstime->tv_nsec - curtime.tv_nsec;
-		flags = EV_ADD;
+		if (nseconds < 0)
+			nseconds = 0;
 	}
 	else
 	{
-		nseconds = 0;
 		flags = EV_DELETE;
+		nseconds = 0;
 	}
 
-	EV_SET(&ev, fd, EVFILT_TIMER, flags, NOTE_NSECONDS, nseconds, NULL);
+	EV_SET(&ev, fd, EVFILT_TIMER, flags, NOTE_NSECONDS, nseconds, &node);
 	return kevent(poller->pfd, &ev, 1, NULL, 0, NULL);
 }
 
@@ -451,10 +457,15 @@ static void __poller_handle_read(struct __poller_node *node,
 		else
 		{
 			nleft = SSL_read(node->data.ssl, p, POLLER_BUFSIZE);
-			if (nleft < 0)
+			if (nleft <= 0)
 			{
 				if (__poller_handle_ssl_error(node, nleft, poller) >= 0)
 					return;
+
+				if (errno == -SSL_ERROR_ZERO_RETURN)
+					nleft = 0;
+				else
+					nleft = -1;
 			}
 		}
 
@@ -499,11 +510,7 @@ static void __poller_handle_read(struct __poller_node *node,
 }
 
 #ifndef IOV_MAX
-# ifdef UIO_MAXIOV
-#  define IOV_MAX	UIO_MAXIOV
-# else
-#  define IOV_MAX	1024
-# endif
+# define IOV_MAX	16
 #endif
 
 static void __poller_handle_write(struct __poller_node *node,
@@ -1059,13 +1066,6 @@ static void *__poller_thread_routine(void *arg)
 		for (i = 0; i < nevents; i++)
 		{
 			node = (struct __poller_node *)__poller_event_data(&events[i]);
-			if (node <= (struct __poller_node *)1)
-			{
-				if (node == (struct __poller_node *)1)
-					has_pipe_event = 1;
-				continue;
-			}
-
 			switch (node->data.operation)
 			{
 			case PD_OP_READ:
@@ -1098,6 +1098,9 @@ static void *__poller_thread_routine(void *arg)
 			case PD_OP_NOTIFY:
 				__poller_handle_notify(node, poller);
 				break;
+			case -1:
+				has_pipe_event = 1;
+				break;
 			}
 		}
 
@@ -1115,11 +1118,16 @@ static void *__poller_thread_routine(void *arg)
 
 static int __poller_open_pipe(poller_t *poller)
 {
+	static struct poller_result node = {
+		.data		=	{
+			.operation	=	-1
+		}
+	};
 	int pipefd[2];
 
 	if (pipe(pipefd) >= 0)
 	{
-		if (__poller_add_fd(pipefd[0], EPOLLIN, (void *)1, poller) >= 0)
+		if (__poller_add_fd(pipefd[0], EPOLLIN, &node, poller) >= 0)
 		{
 			poller->pipe_rd = pipefd[0];
 			poller->pipe_wr = pipefd[1];
@@ -1600,7 +1608,6 @@ int poller_add_timer(const struct timespec *value, void *context, void **timer,
 int poller_del_timer(void *timer, poller_t *poller)
 {
 	struct __poller_node *node = (struct __poller_node *)timer;
-	int stopped = 0;
 
 	pthread_mutex_lock(&poller->mutex);
 	if (!node->removed)
@@ -1614,9 +1621,6 @@ int poller_del_timer(void *timer, poller_t *poller)
 
 		node->error = 0;
 		node->state = PR_ST_DELETED;
-		stopped = poller->stopped;
-		if (!stopped)
-			write(poller->pipe_wr, &node, sizeof (void *));
 	}
 	else
 	{
@@ -1625,7 +1629,7 @@ int poller_del_timer(void *timer, poller_t *poller)
 	}
 
 	pthread_mutex_unlock(&poller->mutex);
-	if (stopped)
+	if (node)
 		poller->callback((struct poller_result *)node, poller->context);
 
 	return -!node;
