@@ -18,6 +18,7 @@
            Liu Kai (liukaidx@sogou-inc.com)
 */
 
+#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <signal.h>
@@ -460,6 +461,8 @@ private:
 			kv.second->deinit();
 			delete kv.second;
 		}
+
+		pthread_rwlock_destroy(&rwlock_);
 	}
 
 private:
